import pika
import json


def rabbitMQ_decorator(function):
    def wrapper(obj, *args, **kwargs):
        assert hasattr(obj, "conn"), "No connection to rabbitMQ server"
        assert hasattr(obj, "channel"), "obj has no property of channel"
        return function(obj, *args, **kwargs)

    return wrapper


class RabbitMQ:
    def conn_and_channel(self, host=None, port=None, virtual_host=None, username=None, password=None):
        credentials = pika.PlainCredentials(username, password)  # mq用户名和密码
        # 虚拟队列需要指定参数 virtual_host，如果是默认的可以不填。
        connection = pika.BlockingConnection(
            pika.ConnectionParameters(host=host, port=port, virtual_host=virtual_host, credentials=credentials)
        )
        channel = connection.channel()
        self.conn = (connection,)
        self.channel = channel
        return connection, channel

    @rabbitMQ_decorator
    def declare_queue_and_bind_dead_letter_queue(
        self,
        exchange,
        queue,
        dead_letter_exchange,
        dead_letter_queue,
        routing_key=None,
        dead_letter_routing_key=None,
        exchange_type="direct",
        dead_letter_exchange_type="direct",
        ttl=None,
        auto_delete=False,
        durable=False,
        dead_letter_auto_delete=False,
        dead_letter_durable=False,
    ):
        """
        创建异常交换器和队列，用于存放没有正常处理的消息。
        :return:
        """
        assert exchange, "exchange is None!!!"
        assert queue, "queue is None!!!"
        # assert routing_key, 'routing_key is None!!!'
        assert exchange_type, "routing_key is None!!!"
        assert dead_letter_exchange, "dead_letter_exchange is None!!!"
        assert dead_letter_queue, "dead_letter_queue is None!!!"
        # assert dead_letter_routing_key, 'dead_letter_routing_key is None!!!'
        assert dead_letter_exchange_type, "routing_key is None!!!"
        assert not (dead_letter_exchange_type != "fanout" and (not dead_letter_routing_key)), (
            "dead_letter_exchange_type = " + dead_letter_exchange_type + " but dead_letter_routing_key is None!!!"
        )
        arguments = {}
        if dead_letter_exchange:
            # 设置死信转发的exchange，延迟结束后指向的交换机(死信收容交换机)
            arguments["x-dead-letter-exchange"] = dead_letter_exchange
        if dead_letter_routing_key:
            arguments["x-dead-letter-routing-key"] = dead_letter_routing_key
        if ttl:
            # 消息的存活时间，消息过期后会被指向(死信收容交换机)收入死信队列
            arguments["x-message-ttl"] = ttl

        self.declare_and_bind(
            dead_letter_exchange,
            dead_letter_queue,
            dead_letter_routing_key,
            exchange_type=dead_letter_exchange_type,
            auto_delete=dead_letter_auto_delete,
            durable=dead_letter_durable,
        )
        self.declare_and_bind(
            exchange,
            queue,
            routing_key,
            exchange_type=exchange_type,
            auto_delete=auto_delete,
            arguments=arguments,
            durable=durable,
        )

    @rabbitMQ_decorator
    def declare_and_bind_ttl_queue(
        self,
        exchange,
        queue,
        ttl=10000,
        routing_key="",
        exchange_type="direct",
        auto_delete=True,
        arguments=None,
        durable=False,
    ):
        assert exchange, "exchange is None!!!"
        assert queue, "queue is None!!!"
        assert ttl, "ttl is None!!!"
        arguments = {}
        arguments["x-message-ttl"] = ttl
        self.declare_and_bind(
            exchange,
            queue,
            routing_key,
            exchange_type=exchange_type,
            auto_delete=auto_delete,
            arguments=arguments,
            durable=durable,
        )

    @rabbitMQ_decorator
    def declare_exchange_and_queue(
        self, exchange, queue, exchange_type="direct", auto_delete=True, arguments=None, durable=False
    ):
        assert exchange, "exchange is None!!!"
        assert queue, "queue is None!!!"
        self.channel.exchange_declare(exchange=exchange, exchange_type=exchange_type)
        self.channel.queue_declare(queue=queue, auto_delete=auto_delete, arguments=arguments, durable=durable)

    @rabbitMQ_decorator
    def declare_and_bind(
        self, exchange, queue, routing_key="", exchange_type="direct", auto_delete=True, arguments=None, durable=False
    ):
        self.declare_exchange_and_queue(
            exchange, queue, exchange_type=exchange_type, auto_delete=auto_delete, arguments=arguments, durable=durable
        )
        self.channel.queue_bind(queue, exchange, routing_key)


class Producter(RabbitMQ):
    @rabbitMQ_decorator
    def basic_publish(self, exchange, routing_key, body):
        body = json.dumps(body) if isinstance(body, dict) or isinstance(body, list) else body
        self.channel.basic_publish(exchange=exchange, routing_key=routing_key, body=body)


class Consumer(RabbitMQ):
    # 定义一个回调函数来处理消息队列中的消息，这里是打印出来
    @rabbitMQ_decorator
    def callback(self, ch, method, properties, body):
        try:
            ch.basic_ack(delivery_tag=method.delivery_tag)
        except Exception as e:
            pass

    @rabbitMQ_decorator
    def start_consume(
        self, queue, on_message_callback, auto_ack=False, exclusive=False, consumer_tag=None, arguments=None
    ):
        self.channel.basic_consume(
            queue,
            on_message_callback,
            auto_ack=auto_ack,
            exclusive=exclusive,
            consumer_tag=consumer_tag,
            arguments=arguments,
        )
        self.channel.start_consuming()



from rest_framework.views import APIView
from common.decorator import METHOD_GET, METHOD_POST, METHOD_PUT, METHOD_DELETE, requests_decorator
from common.logging_config import logger
from common.rest_extend.response import Results, UPSTREAM_SERVER_ERROR_CODE, RESTResponse
from common.rest_extend.decorator import vaild_decorator

DEFAULT_ES_URL = "config.esserver.eshosturl"
DEFAULT_INDEX = "config.esserver.pointtableindex"
DEFAULT_ESSERVER_VERSION = "config.esserver.version"


class ESRequests:
    ES_URL = DEFAULT_ES_URL
    INDEX = DEFAULT_INDEX
    BASE_ES_URL = "{host}?index={index}".format(host=ES_URL, index=INDEX)
    ESSERVER_VERSION = DEFAULT_ESSERVER_VERSION

    def __new__(cls, *args, **kwargs):
        ES_URL = kwargs.pop("ES_URL", None)
        INDEX = kwargs.pop("INDEX", None)
        ESSERVER_VERSION = kwargs.pop("ESSERVER_VERSION", None)
        if ES_URL:
            cls.ES_URL = ES_URL
        if INDEX:
            cls.INDEX = INDEX
        if ESSERVER_VERSION:
            cls.ESSERVER_VERSION = ESSERVER_VERSION
        # assert (
        #     cls.ES_URL == DEFAULT_ES_URL
        #     or cls.INDEX == DEFAULT_INDEX
        #     or cls.ESSERVER_VERSION == DEFAULT_ESSERVER_VERSION
        # ), f"'ES_URL'={ES_URL} or 'INDEX'={INDEX} or 'DEFAULT_ESSERVER_VERSION'={DEFAULT_ESSERVER_VERSION}"

        cls.BASE_ES_URL = "{host}?index={index}".format(host=cls.ES_URL, index=cls.INDEX)
        logger.info(cls.ES_URL, cls.INDEX, cls.ESSERVER_VERSION, cls.BASE_ES_URL)
        return super().__new__(cls, *args, **kwargs)

    def get(self, _id=None, data: dict = None):
        """

        :param _id:
        :param data:
        :return:
        """
        if _id:
            url = "{base_url}&id={id}".format(base_url=self.BASE_ES_URL, id=_id)
        else:
            url = self.BASE_ES_URL
        if not data:
            data = {}

        data = {"version": self.ESSERVER_VERSION, "data": data}
        results, response = self.get_requests_response(url, METHOD_GET, data)
        if results.code == 200:
            data = response.json()
            data = data.get("data")
            results.data = data

        return results

    def post(self, data, _id=None):
        """

        :param data:
        :param _id:
        :return:
        """
        if _id:
            url = "{base_url}&id={id}".format(base_url=self.BASE_ES_URL, id=_id)
        else:
            url = self.BASE_ES_URL
        body = {"version": self.ESSERVER_VERSION, "data": data}
        results, response = self.get_requests_response(url, METHOD_POST, body)
        return results

    def put(self, _id, data):
        """
        :param _id:
        :param data:
        :return:
        """
        url = "{base_url}&id={id}".format(base_url=self.BASE_ES_URL, id=_id)
        body = {"version": self.ESSERVER_VERSION, "data": data}

        results, response = self.get_requests_response(url, METHOD_PUT, body)
        return results

    def delete(self, _id):
        """
        :param _id:
        :return:
        """
        url = "{base_url}&id={id}".format(base_url=self.BASE_ES_URL, id=_id)
        body = {"version": self.ESSERVER_VERSION, "data": {}}
        results, response = self.get_requests_response(url, METHOD_DELETE, body)
        return results

    @requests_decorator(failure_skip=False)
    def download(self, url, *args):
        response = args[0]
        return response

    def get_requests_response(self, url, method, data=None) -> tuple:
        """
        请求es接口，并进行初步判断
        :param url:
        :param method:
        :param data:
        :return: tuple-》results，response
        """
        results = Results()
        # if isinstance(data, dict) or isinstance(data, list):
        #     data = json.dumps(data)
        response = None
        try:
            response = self.download(url, method=method, json=data)
            if response != None:
                if response.status_code >= 500:
                    results.code = UPSTREAM_SERVER_ERROR_CODE
                else:
                    results.code = response.status_code
                results.describe = response.json().get("describe")
            else:
                results.code = 500
                results.describe = "download error"

                # data = response.json()
                # describe = data.get("describe")
                # # 判断上层服务是否正常响应
                # if response.status_code == 400 and (
                #         "pointtable not found  id:" in describe or "删除一个不存在" in describe
                # ):
                #     results.code = 200
                # else:
                #     results.code = UPSTREAM_SERVER_ERROR_CODE
                #     results.describe = describe

        except Exception as e:
            results.code = UPSTREAM_SERVER_ERROR_CODE
            results.describe = str(e)
        return results, response


class ESView(APIView):
    tools: ESRequests = None

    def get(self, request, obj_ser=None, data=None):
        _id = request.GET.get("id")
        if not data:
            _data = data
        else:
            _data = request.data.get("data")
        results = self.tools.get(_id, _data)

        return RESTResponse(results)

    def post(self, request, obj_ser=None):
        results = self.tools.post(obj_ser.validated_data.get("data"))
        return RESTResponse(results)

    def put(self, request, obj_ser=None, data=None, need_results=False):
        _id = request.GET.get("id")
        results = self.validate_id(_id)
        if not results:
            results = self.tools.put(_id=_id, data=obj_ser.validated_data.get("data"))
        return RESTResponse(results)

    def delete(self, request, obj_ser=None, data=None, need_results=False):
        _id = request.GET.get("id")
        results = self.validate_id(_id)
        if not results:
            results = self.tools.delete(_id)
        return RESTResponse(results)

    def validate_id(self, _id):
        results = Results()
        if not _id:
            results.describe = "id cannot be empty!!!"
            return results

    def vaild_data(self, request, func, obj_ser):
        external_wrapper = vaild_decorator(obj_serializer=obj_ser)
        internal_wrapper = external_wrapper(func)
        res = internal_wrapper(self, request)
        print(res)



from rest_framework import serializers

from tenant_auth.models import AuthTenant


class TenantSerializer(serializers.ModelSerializer):
    class Meta:
        model = AuthTenant
        # fields = '__all__'
        exclude = ("password",)



import json
import uuid
import paho.mqtt.client as mqtt
from common.logging_config import logger
from common.utility.base import get_uuid


def mqtt_decorator(function):
    def wrapper(obj, *args, **kwargs):
        assert hasattr(obj, "client"), "No connection to mqtt server"
        return function(obj, *args, **kwargs)

    return wrapper


class MQTTClient:
    # _instance_lock = threading.Lock()
    #
    # def __new__(cls, *args, **kwargs):
    #     if not hasattr(cls, '_instance'):
    #         with MQTTClient._instance_lock:
    #             if not hasattr(cls, '_instance'):
    #                 MQTTClient._instance = super().__new__(cls)
    #
    #         return MQTTClient._instance
    #     else:
    #         raise Exception('不能实例化多个类')

    def __init__(self, on_connect=None, on_message=None, on_subscribe=None, resub=True):
        self._on_connect = on_connect if on_connect else self.on_connect
        self._on_message = on_message if on_message else self.on_message
        self._on_subscribe = on_subscribe if on_subscribe else self.on_subscribe
        self.sub_topics = []
        self.resub = resub
        self._obj_init = True

    @mqtt_decorator
    def mqtt_publish(self, topic, data):
        if isinstance(data, dict) or isinstance(data, list):
            data = json.dumps(data)
        self.client.publish(topic, payload=data, qos=0)

    @mqtt_decorator
    def mqtt_subscribe(self, topic):
        self.client.subscribe(topic)
        self.sub_topics.append(topic)
        logger.debug("subscribe topic", topic)

    def on_connect(self, client, userdata, flags, rc):
        if self.resub and self._obj_init is False:
            for topic in self.sub_topics:
                self.client.subscribe(topic)
            logger.debug("resub topic ", self.sub_topics)
        self._obj_init = False
        logger.debug("Connected with result code: " + str(rc))

    def on_message(self, client, userdata, msg):
        logger.debug(msg.topic + " " + str(msg.payload))

    def on_subscribe(self, client, userdata, mid, granted_qos):
        logger.debug("On Subscribed: qos = {qos}".format(qos=granted_qos))

    def connect_mtqq(
        self, host=None, port=2883, client_id="", username=None, password=None, keepalive=600, describe=""
    ):
        if not client_id:
            client_id = get_uuid()
        logger.info(host, port, client_id, keepalive, describe)
        client = mqtt.Client(client_id)
        client.username_pw_set(username=username, password=password)
        client.on_connect = self._on_connect
        client.on_message = self._on_message
        client.on_subscribe = self._on_subscribe
        client.connect(host, port, keepalive)  # 600为keepalive的时间间隔
        self.client = client
        self.loop_forever()
        return client

    def loop_forever(self):
        from threading import Thread

        Thread(target=self.start).start()
        logger.debug("setting mqtt loop_forever")

    def start(self):
        self.client.loop_forever()  # 保持连接

    def get_uuid(self):
        uid = str(uuid.uuid4())
        suid = "".join(uid.split("-"))
        return suid


if __name__ == "__main__":
    mqtt_client = MQTTClient()
    # mqtt_client.connect_mtqq(
    #     host=config.agentmqtt.server,
    #     port=config.agentmqtt.port,
    #     username=config.agentmqtt.username,
    #     password=config.agentmqtt.password,
    #     describe="norm series  start norm series",
    # )
    mqtt_client.mqtt_subscribe("sd")



from datetime import datetime
from rest_framework import serializers


def iot_date_validate_decorator(function):
    def wrapper(obj, data, *args, **kwargs):
        dateTo = data.get("dateTo")
        dateFrom = data.get("dateFrom")
        try:

            if "." in dateTo:
                dateTo = datetime.strptime(dateTo, "%Y-%m-%dT%H:%M:%S.%fZ")
            else:
                dateTo = datetime.strptime(dateTo, "%Y-%m-%dT%H:%M:%SZ")
        except Exception as e:
            raise serializers.ValidationError("dateTo： " + str(e))
        try:
            if "." in dateFrom:
                dateFrom = datetime.strptime(dateFrom, "%Y-%m-%dT%H:%M:%S.%fZ")
            else:
                dateFrom = datetime.strptime(dateFrom, "%Y-%m-%dT%H:%M:%SZ")
        except Exception as e:
            raise serializers.ValidationError("dateFrom： " + str(e))
        if dateTo < dateFrom:
            raise serializers.ValidationError("dateTo 需大于  dateFrom")

        return function(obj, data, *args, **kwargs)

    return wrapper


class BaseParamSerializer(serializers.Serializer):
    version = serializers.CharField(required=True, allow_blank=True)
    externalid = serializers.CharField(required=True, allow_blank=True)
    sn = serializers.CharField(required=True, allow_blank=True)
    ops = serializers.CharField(required=True, allow_blank=True)
    data = serializers.DictField(required=True)



import time
import uuid
from datetime import datetime


def dt_time():
    nnn = time.time() - 8 * 60 * 60
    dat = datetime.fromtimestamp(nnn)
    ttt = dat.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    return ttt


def dt_now_time():
    nnn = time.time()
    dat = datetime.fromtimestamp(nnn)
    ttt = dat.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    return ttt


def get_uuid():
    uid = str(uuid.uuid4())
    suid = "".join(uid.split("-"))
    return suid



class BaseArgsParsing:
    PAGE = "page"
    SIZE = "size"
    PAGESIZE = "pagesize"
    SORT = "sort"
    ASC = "asc"
    DESC = "desc"

    LIKE = "like"
    NOT = "not"
    IN = "in"
    LT = "lt"
    GT = "gt"
    LTE = "lte"
    GTE = "gte"

    RESERVED_FIELD = [PAGE, SIZE, PAGESIZE, SORT]
    SUFFIX_KEYWORD = [LIKE, NOT, IN, LT, GT, LTE, GTE]

    def get_page_and_size(self, data):
        """
        获取页面和条数
        :param data:
        :return:
        """
        try:
            page = int(data.get(self.PAGE, 1))
            size = int(data.get(self.SIZE, 20))
            if data.get(self.PAGESIZE):
                size = int(data.get(self.PAGESIZE))
        except ValueError as e:
            raise ValueError(f"'{self.PAGE}' or '{self.SIZE}' must be int")
        return page, size

    def get_sort(self, data, default_field=None, default_sort=None):
        item = data.get(self.SORT)
        field = default_field
        sort = default_sort
        assert (default_field and default_sort) or (
            (not default_field) and (not default_sort)
        ), f"default_field={default_field}, default_sort={default_sort}"
        if item:
            item = item.lower().strip().split(" ")
            if len(item) <= 1:
                raise ValueError(f"'{self.SORT}'  format like  'sort=field asc'")
            field = item[0]
            sort = item[-1].lower()
            if sort != self.ASC and sort != self.DESC:
                raise ValueError(f"'{self.SORT}' must be  '{self.ASC}'  or '{self.DESC}'")
        return field, sort



from dataclasses import asdict
from django.http import HttpResponse, JsonResponse
from django.utils.deprecation import MiddlewareMixin
import json
from common.logging_config import get_my_logger, DOCUMENTPATH
from common.rest_extend.response import Results, REQUEST_ERROR_CODE, FORBID_CODE, UNAUTHORIZED

logger = get_my_logger(DOCUMENTPATH, "django")


def record_log(request, response=None):
    """
    日志记录
    :param request:
    :param response:
    :return:
    """
    if request.META.get("HTTP_X_FORWARDED_FOR"):
        ip = request.META.get("HTTP_X_FORWARDED_FOR")
    else:
        ip = request.META.get("REMOTE_ADDR")

    method = request.method
    path = request.path
    args = json.dumps(request.GET) if request.GET else ""
    try:
        body = json.dumps(json.loads(request.body)) if request.body else ""
    except Exception as e:
        logger.exception(e)
        body = request.body
        # return True
    if response:
        try:
            content = json.dumps(json.loads(response.content))

        except Exception as e:
            pass
            content = response.content
        logger.info("process_response", ip, method, path, args, body, content)
    else:
        logger.info("process_request", ip, method, path, args, body)


class RecordMiddleware(MiddlewareMixin):
    def process_request(self, request):
        result = record_log(request)
        if result:
            results = Results()
            results.describe = "error data  " + str(request.body)
            results.code = REQUEST_ERROR_CODE
            return JsonResponse(asdict(results), status=results.code)

    def process_exception(self, request, exception):
        return HttpResponse(exception, status=500)

    def process_response(self, request, response):
        result = record_log(request, response)
        # if result:
        #     results = Results()
        #     results.describe = 'error data  ' + str(request.body)
        #     results.code = REQUEST_ERROR_CODE
        #     return JsonResponse({}, status=results.code)

        return response



import abc
import datetime
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Q, Count
from django.db.models.query import QuerySet
from rest_framework.exceptions import ValidationError
from rest_framework.views import APIView
from copy import deepcopy

from common.rest_extend.decorator import relevance_vaild_legal_decorator
from common.rest_extend.response import (
    Results,
    SUCCESS,
    FAILURE,
    REQUEST_ERROR_CODE,
    SERVER_ERROR_CODE,
    PageTypeResults,
    get_error_status_code, RESTResponse,
)
from dataclasses import asdict, is_dataclass

# from django.core.paginator import Paginator, Page
from common.utility.args_parsing import BaseArgsParsing


class ResponseJsonEncoder(DjangoJSONEncoder):
    def default(self, o):
        _o = super(ResponseJsonEncoder, self).default(o)
        if isinstance(o, datetime.datetime):

            return o.strftime("%Y-%m-%d %H:%M:%S")
        else:
            return _o


class Select(BaseArgsParsing, metaclass=abc.ABCMeta):
    GROUP = "group"

    RESERVED_FIELD = deepcopy(BaseArgsParsing.RESERVED_FIELD) + [GROUP]
    FORBID_VALUES = {GROUP: ["password"]}

    def get_orm_q(self, data):
        """
        获取筛选条件
        :param data:
        :return:
        """
        orm_q = Q()
        for key in data:
            if (not key) or (not data.get(key)):
                continue
            values = data.get(key)
            if key in self.FORBID_VALUES.keys():
                if any(item in values for item in self.FORBID_VALUES[key]):
                    raise ValueError(f" {key}  value cannot be a {values}")

            # 判断预留字段不当查询条件
            if key in self.RESERVED_FIELD:
                continue
            itme = key.split("_")
            if len(itme) > 1:
                real_key = "_".join(itme[:-1])
                keyword = itme[-1].lower()
                if keyword in self.SUFFIX_KEYWORD:
                    if keyword == self.LIKE:
                        orm_q.add(Q(**{real_key + "__contains": data.get(key)}), Q.AND)
                    elif keyword == self.IN:
                        values = data.get(key).split(",")
                        orm_q.add(Q(**{real_key + "__in": values}), Q.AND)
                    elif keyword == self.NOT:
                        orm_q.add(~Q(**{real_key: data.get(key)}), Q.AND)
                    else:
                        orm_q.add(Q(**{real_key + '__' + keyword: values}), Q.AND)
                    continue

            orm_q.add(Q(**{key: data.get(key)}), Q.AND)

        return orm_q

    def get_page_and_size(self, data):
        """
        获取页面和条数
        :param data:
        :return:
        """
        try:
            page = int(data.get(self.PAGE, 1))
            size = int(data.get(self.SIZE, 20))
            if data.get(self.PAGESIZE):
                size = int(data.get(self.PAGESIZE))
        except ValueError as e:
            raise ValueError(f"'{self.PAGE}' or '{self.SIZE}' must be int")
        return page, size

    def get_page_type_results(self, result, total, page, size, *args, **kwargs):
        """
        获取当前页面结果集
        :param result:
        :param total:
        :param page:
        :param size:
        :param args:
        :param kwargs:
        :return:
        """
        page_type_results = PageTypeResults()
        # page_type_results.group = group_result
        page_type_results.result = result
        page_type_results.total = total
        page_type_results.page = page
        page_type_results.size = size
        return page_type_results

    def paging(self, queryset, data, obj_serializer=None):
        """
        分页
        :param queryset:
        :param data:
        :return:
        """
        total = len(queryset)
        page, size = self.get_page_and_size(data)
        start = (page - 1) * size
        end = page * size
        current_page_queryset = queryset[start:end]
        current_page_result = []
        if current_page_queryset:
            # 如果是group  queryset是dict类型
            if not isinstance(current_page_queryset[0], dict):
                if obj_serializer:
                    current_page_result = list(obj_serializer(current_page_queryset, many=True).data)
                else:
                    current_page_result = list(queryset.values()[start:end])
            else:
                current_page_result = current_page_queryset
        return current_page_queryset, current_page_result, total, page, size

    def select(self, obj, data):
        """
        执行查询
        :param obj:
        :param data:
        :return:
        """
        orm_q = self.get_orm_q(data)
        queryset = obj.objects.select_related().filter(orm_q).all()
        return queryset

    def sort_queryset(self, data, queryset):
        """
        排序
        :param data:
        :param queryset:
        :return:
        """
        field, sort = self.get_sort(data, default_field='id', default_sort=self.DESC)
        queryset = queryset.order_by(field if Select.ASC == sort else f"-{field}")

        return queryset

    def group(self, obj, data):
        """
        group
        :param obj:
        :param data:
        :return:
        """
        orm_q = self.get_orm_q(data)
        group = data.get(self.GROUP)
        if group:
            group = tuple(group.split(","))
            queryset = obj.objects.select_related().filter(orm_q).all()
            queryset = queryset.values(*group).annotate(count=Count("id"))
        else:
            raise Exception(f"'{self.GROUP}'  is None!!!")
        return queryset

    def find(self, obj, data, obj_serializer=None):
        """
        查询、并返回Results
        1、获取queryset
        2、排序
        3、分页
        4、获取当前页结果
        5、扩展当前页结果
        :param obj:
        :param data:
        :return:
        """
        results = Results()
        queryset = None
        current_page_queryset = None
        try:
            # 获取queryset ；extend_queryset用于扩展页面数据集
            queryset, extend_queryset = self.get_queryset(obj, data)
            queryset = self.sort_queryset(data, queryset)
            current_page_queryset, current_page_result, total, page, size = self.paging(queryset, data, obj_serializer)
            page_type_results = self.get_page_type_results(current_page_result, total, page, size)
            self.extend_page_type_results(data, page_type_results, extend_queryset)
            results.data = page_type_results
            results.data_length = len(page_type_results.result)
            results.code = 200
        except Exception as e:
            results.code, results.describe = get_error_status_code(e)
        return results, queryset, current_page_queryset

    @abc.abstractmethod
    def get_queryset(self, obj, data) -> (QuerySet, QuerySet):
        group = data.get(self.GROUP)
        if group:
            queryset = self.group(obj, data)
        else:
            queryset = self.select(obj, data)
        return queryset, None

    def extend_page_type_results(self, data, page_type_results, extend_queryset, *args, **kwargs):
        pass


class BaseView(APIView, Select):
    """
    对实体模型增删改查的父类
    """

    def get(self, request, obj, obj_serializer, data=None, need_results=False, need_queryset=False, keys=None):
        """
        根据键值对查找数据
        :param request:
        :param obj:
        :param obj_serializer:
        :param data:
        :param need_results:
        :param keys:
        :return:
        """
        results, queryset, current_page_queryset = self.find(obj, request.GET, obj_serializer)
        if need_results:
            return results
        if need_queryset:
            return results, queryset
        return RESTResponse(results)

    def post(self, request, obj, obj_serializer, data=None, need_results=False, **kwargs):
        """
        添加数据
        :param request:
        :param obj:
        :param obj_serializer:
        :param data:
        :param need_results:
        :return:
        """
        results = Results()
        if not data:
            data = request.data
        if data.get('data'):
            data = data.get('data')
        serializer = obj_serializer(data=data)
        try:
            valid = serializer.is_valid(raise_exception=True)
            serializer.save()
            results.describe = "add  successfully！！！"
            results.status = SUCCESS
            results.code = 200
        except Exception as e:
            results.code, results.describe = get_error_status_code(e)

        if need_results:
            return results
        return RESTResponse(results)

    def put(self, request, obj, obj_serializer, data=None, need_results=False, pk="id", **kwargs):
        """
        根据主键，或某个字段更新数据
        :param request:
        :param obj:
        :param obj_serializer:
        :param data:
        :param need_results:
        :param pk:
        :return:
        """
        results = Results()
        results.code = 200
        if not data:
            data = request.data
        if data.get('data'):
            data = data.get('data')
        partial = True
        if request.GET.get("pk"):
            pk = request.GET.get("pk")
        try:
            value_pk = data.get(pk)
            if not value_pk:
                results.describe = "'pk' cannot be empty"
                results.code = REQUEST_ERROR_CODE

            else:
                if pk == "id":
                    try:
                        value_pk = int(value_pk)
                    except:
                        raise ValidationError("'pk' type error Should be <int>！！！")

                instance = obj.objects.filter(**{pk: value_pk}).first()
                if not instance:
                    results.describe = pk + "=" + str(value_pk) + "  does not exist"
                    results.code = REQUEST_ERROR_CODE
                else:

                    serializer = obj_serializer(instance, data=data, partial=partial)
                    try:
                        serializer.is_valid(raise_exception=True)
                        serializer.save()
                        results.describe = "update  successfully！！！"
                        results.status = SUCCESS
                    except Exception as e:
                        results.code, results.describe = get_error_status_code(e)
                    if getattr(instance, "_prefetched_objects_cache", None):
                        # If 'prefetch_related' has been applied to a queryset, we need to
                        # forcibly invalidate the prefetch cache on the instance.
                        instance._prefetched_objects_cache = {}

        except Exception as e:
            results.code, results.describe = get_error_status_code(e)

        if need_results:
            return results
        return RESTResponse(results)

    def delete(self, request, obj, obj_serializer, data=None, need_results=False, pk="id", **kwargs):
        """
        根据主键删除数据
        :param request:
        :param obj:
        :param obj_serializer:
        :param data:
        :param need_results:
        :param pk:
        :return:
        """
        results = Results()
        results.code = 200
        if not data:
            data = request.data
        if data.get('data'):
            data = data.get('data')
        if request.GET.get("pk"):
            pk = request.GET.get("pk")
        value_pk = data.get(pk)
        if not value_pk:
            results.describe = "'pk' cannot be empty"
            results.code = REQUEST_ERROR_CODE
        else:
            try:
                if pk == "id":
                    try:
                        value_pk = int(value_pk)
                    except:
                        raise ValidationError("'pk' type error Should be <int>！！！")

                instance = obj.objects.filter(id=value_pk).first()
                if not instance:
                    results.describe = pk + "=" + str(value_pk) + " does not exist"
                    results.code = REQUEST_ERROR_CODE
                else:
                    instance.delete()
                    results.describe = "deleted successfully！！！"
                    results.status = SUCCESS
            except Exception as e:
                results.code, results.describe = get_error_status_code(e)

        if need_results:
            return results
        return RESTResponse(results)

    def get_queryset(self, obj, data) -> (QuerySet, QuerySet):
        return super(BaseView, self).get_queryset(obj, data)


class BaseJoinView(APIView, Select):
    CHOOSE_GROUP = "choosegroup"
    RESERVED_FIELD = deepcopy(Select.RESERVED_FIELD) + [CHOOSE_GROUP]
    FORBID_VALUES = dict(**Select.FORBID_VALUES, **{CHOOSE_GROUP: ["password"]})

    @relevance_vaild_legal_decorator
    def get(self, request, obj, obj_serializer, obj_relevance=None, m_to_m_obj=None, mapping: dict = None,
            m_to_m_mapping=None):
        """
        扩展返回字段（关联表）
        1、获取queryset
        2、扩展字段
        :param request:
        :param obj:
        :param obj_serializer:
        :param obj_relevance: 外键关联对象
        :param mapping: 需要扩展字段对应的映射
        :return:
        """
        results, queryset, current_page_queryset = self.find(obj, request.GET, obj_serializer)
        if results.data and results.data.result and mapping and obj_relevance:
            result = self.extend_result_field(obj_serializer, obj_relevance, m_to_m_obj, mapping, m_to_m_mapping,
                                              current_page_queryset)
            results.data.result = result
        return RESTResponse(results)

    def extend_result_field(self, obj_serializer, obj_relevance, m_to_m_obj, mapping, m_to_m_mapping, queryset):
        """
        扩展字段
        1、获取关联表
        2、遍历结果集、进行字段扩展
        :param obj_relevance:
        :param mapping:
        :param queryset:
        :return:
        """

        result = []
        # 遍历结果集
        for item in queryset:
            item_result = item if isinstance(item, dict) else dict(obj_serializer(item).data)
            self.extend_m_to_m_field(item, item_result, m_to_m_obj, m_to_m_mapping)
            self.extend_relevance_field(item, item_result, obj_relevance, mapping)
            result.append(item_result)
        return result

    def extend_m_to_m_field(self, item, item_result, m_to_m_obj, m_to_m_mapping):
        if m_to_m_obj:
            m_to_m_name = m_to_m_obj.__name__.lower()
            item_result[m_to_m_name] = []
            m_to_m = getattr(item, m_to_m_name).all()
            for m_to_m_obj in m_to_m:
                for key in m_to_m_mapping:
                    value = getattr(m_to_m_obj, m_to_m_mapping[key])
                    if value:
                        item_result[key].append(value)

    def extend_relevance_field(self, item, item_result, obj_relevance, mapping):
        # 获取关联表
        relevance_name = obj_relevance.__name__.lower()
        # 获取queryset中的外键对象
        _obj = getattr(item, relevance_name)
        # 对结果集进行字段扩展
        for key in mapping:
            child_obj = _obj
            if child_obj:
                # 多级 关联
                if "." in mapping[key]:
                    attrs = mapping[key].split(".")
                    # 多级 关联的外键对象
                    child_obj = getattr(_obj, attrs[0])
                    for attr in attrs[1:]:
                        child_obj = getattr(child_obj, attr)
                    item_result[key] = child_obj
                else:
                    item_result[key] = getattr(_obj, mapping[key], None)
            else:
                item_result[key] = None

    def get_queryset(self, obj, data) -> (QuerySet, QuerySet):

        drop_group = data.get(self.CHOOSE_GROUP)
        queryset, extend_queryset = super().get_queryset(obj, data)
        if drop_group:
            group = tuple(drop_group.split(","))
            extend_queryset = queryset.values(*group).annotate(count=Count("id"))
        return queryset, extend_queryset

    # def get_page_type_results(self, result, total, page, size, group_result=None, **kwargs):
    #     if not group_result:
    #         group_result = {}
    #     page_type_results = super().get_page_type_results(result, total, page, size)
    #     page_type_results.group = group_result
    #     return page_type_results

    def extend_page_type_results(self, data, page_type_results, extend_queryset, *args, **kwargs):
        if extend_queryset:
            page_type_results.group = list(extend_queryset)



from concurrent.futures.thread import ThreadPoolExecutor

import requests
from requests.exceptions import ConnectionError as ConnError


def thread_pool_decorator(max_workers=10):
    pool = ThreadPoolExecutor(max_workers=max_workers)

    def external_wrapper(function):
        def internal_wrapper(*args, **kwargs):
            task = pool.submit(function, *args, **kwargs)
            # return task.result()

        return internal_wrapper

    return external_wrapper


METHOD_POST = "POST"
METHOD_GET = "GET"
METHOD_PUT = "PUT"
METHOD_DELETE = "DELETE"

DEFAULT_STOP_CODE = [403, 404, 400, 500]
DEFAULT_SUCCESS_CODE = [200, 201]


def requests_decorator(method=METHOD_GET, success_code=200, stop_code=None, failure_skip=True, max_retry=3, timeout=30):
    def external_wrapper(function):
        def internal_wrapper(*args, **kwargs):
            _method = kwargs.pop("method", None)
            if not _method:
                _method = method
            _method = _method.upper()
            if _method != METHOD_GET and _method != METHOD_POST and _method != METHOD_PUT and _method != METHOD_DELETE:
                raise ValueError(str(_method) + "  request mode is not supported")
            data = kwargs.pop("data", None)
            if data:
                if not isinstance(data, str):
                    raise ValueError("  data type must bu str")

            re_try = 0
            obj = not isinstance(args[0], str)
            url = args[1] if obj else args[0]
            _stop_code = DEFAULT_STOP_CODE
            if stop_code:
                if isinstance(stop_code, list):
                    _stop_code = DEFAULT_STOP_CODE + stop_code
            while re_try < max_retry:
                try:
                    re_try += 1
                    response = requests.request(_method, url, timeout=timeout, **kwargs)

                    if any(
                            [
                                response.status_code == success_code,
                                response.status_code in DEFAULT_SUCCESS_CODE,
                                response.status_code in _stop_code,
                            ]
                    ):
                        break
                except ConnectionError as e:
                    raise ConnectionError(e)
                except ConnError as e:
                    raise ConnError(e)
                except TypeError as e:
                    raise TypeError(e)
                except Exception as e:
                    pass
            else:
                return
            if response.status_code in _stop_code and failure_skip:
                return
            if obj:
                return function(args[0], args[1], response, *args[2:])
            else:
                return function(args[0], response, *args[1:])

        return internal_wrapper

    return external_wrapper


@requests_decorator()
def download(url, *args):
    response = args[0]
    return response



from django.db import models


class BaseModels(models.Model):
    class Meta:
        abstract = True

    creator = models.CharField(max_length=16, help_text="创建人", null=True)
    addon = models.DateTimeField(auto_now_add=True, help_text="添加时间", editable=True)
    update = models.DateTimeField(auto_now=True, help_text="最后更新时间", editable=True)



import os
from logging.handlers import RotatingFileHandler
import logging
from pathlib import Path
from cmreslogging.handlers import CMRESHandler

BASEPATH = os.getcwd()
DOCUMENT_PATH_NAME = "document"
# 文档存放路径
DOCUMENTPATH = os.path.join(BASEPATH, DOCUMENT_PATH_NAME)

SUPER_FUNC = "super_func"


def get_my_logger(file_path, file_name, logger_name=__name__, **kwargs):
    """
    获取日志记录器
    :param log_path: 日志保存位置
    :param logger_name:
    :param kwargs:
    :return:
    """
    parent_path = file_path
    while not Path(file_path).exists():
        print("reset log file path!!!", file_path)
        # if not os.path.exists(file_path):
        #     os.mkdir(file_path)
        parent_path = os.path.abspath(os.path.join(parent_path, ".."))
        file_path = os.path.join(parent_path, DOCUMENT_PATH_NAME)

    file_path = os.path.join(file_path, file_name)
    print(file_path)

    logger = getLogger(file_path)
    logger.setLevel(level=logging.DEBUG)
    out_formatter = "%(levelname)s %(filename)s:%(lineno)d %(asctime)s  %(message)s"
    # 设置输出控制台
    stream_handler = logging.StreamHandler()
    stream_formatter = logging.Formatter(out_formatter, datefmt="%Y-%m-%d %H:%M:%S")
    stream_handler.setFormatter(stream_formatter)
    logger.addHandler(stream_handler)

    # 设置日志文件输出，INFO级别
    info_rt_handler = RotatingFileHandler(
        filename=file_path + ".info", mode="a", maxBytes=100 * 1024 * 1024, backupCount=3, encoding="utf-8"
    )
    info_rt_handler.setLevel(level=logging.INFO)

    # 设置日志文件输出，WARNING级别
    warnig_rt_handler = RotatingFileHandler(
        filename=file_path + ".warning", mode="a", maxBytes=100 * 1024 * 1024, backupCount=3, encoding="utf-8"
    )
    warnig_rt_handler.setLevel(level=logging.WARNING)

    debug_rt_handler = RotatingFileHandler(
        filename=file_path + ".debug", mode="a", maxBytes=100 * 1024 * 1024, backupCount=3, encoding="utf-8"
    )
    debug_rt_handler.setLevel(level=logging.DEBUG)

    error_rt_handler = RotatingFileHandler(
        filename=file_path + ".error", mode="a", maxBytes=100 * 1024 * 1024, backupCount=3, encoding="utf-8"
    )
    error_rt_handler.setLevel(level=logging.ERROR)

    critical_rt_handler = RotatingFileHandler(
        filename=file_path + ".critical", mode="a", maxBytes=100 * 1024 * 1024, backupCount=3, encoding="utf-8"
    )
    critical_rt_handler.setLevel(level=logging.CRITICAL)

    # 设置打印格式
    file_rt_formatter = logging.Formatter(out_formatter, datefmt="%Y-%m-%d %H:%M:%S")

    info_rt_handler.setFormatter(file_rt_formatter)
    warnig_rt_handler.setFormatter(file_rt_formatter)
    debug_rt_handler.setFormatter(file_rt_formatter)
    error_rt_handler.setFormatter(file_rt_formatter)
    critical_rt_handler.setFormatter(file_rt_formatter)

    logger.addHandler(info_rt_handler)
    logger.addHandler(warnig_rt_handler)
    logger.addHandler(error_rt_handler)
    logger.addHandler(debug_rt_handler)
    logger.addHandler(critical_rt_handler)

    handler = CMRESHandler(
        hosts=[{"host": "192.168.111.112", "port": 9200}],
        auth_type=CMRESHandler.AuthType.NO_AUTH,
        es_index_name="python-my_tests-my-index",
    )

    logger.addHandler(handler)
    return logger


def getLogger(name=None):
    """
    Return a logger with the specified name, creating it if necessary.

    If no name is specified, return the root logger.
    """
    if name:
        MyLogger.manager.loggerClass = MyLogger
        return MyLogger.manager.getLogger(name)
    else:
        raise Exception("name is None!!!")


def logger_decorator(func):
    def wrapper(obj, msg, *args, **kwargs):
        if SUPER_FUNC in args:
            args = list(args)
            args.remove(SUPER_FUNC)
            return func(obj, msg, *args, **kwargs)

        else:
            msg = str(msg)
            msg += "  " + "  ".join([str(m) for m in args])
            return func(obj, msg, (), **kwargs)

    return wrapper


def reset_msg(msg, *args):
    try:
        if SUPER_FUNC in args:
            args = list(args)
            args.remove(SUPER_FUNC)
            return msg, args

        else:
            msg = str(msg)
            if args:
                msg += "   --   " + "   --   ".join([str(m) for m in args])
            return msg, ()
    except Exception as e:
        return msg, args


class LoggerTemplate(logging.Logger):
    def debug(self, msg, *args, **kwargs):
        """
        Log 'msg % args' with severity 'DEBUG'.

        To pass exception information, use the keyword argument exc_info with
        a true value, e.g.

        logger.debug("Houston, we have a %s", "thorny problem", exc_info=1)
        """
        msg, args = reset_msg(msg, *args)
        if self.isEnabledFor(logging.DEBUG):
            self._log(logging.DEBUG, msg, args, **kwargs)

    def info(self, msg, *args, **kwargs):
        """
        Log 'msg % args' with severity 'INFO'.

        To pass exception information, use the keyword argument exc_info with
        a true value, e.g.

        logger.info("Houston, we have a %s", "interesting problem", exc_info=1)
        """
        msg, args = reset_msg(msg, *args)
        if self.isEnabledFor(logging.INFO):
            self._log(logging.INFO, msg, args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        """
        Log 'msg % args' with severity 'WARNING'.

        To pass exception information, use the keyword argument exc_info with
        a true value, e.g.

        logger.warning("Houston, we have a %s", "bit of a problem", exc_info=1)
        """

        msg, args = reset_msg(msg, *args)
        if self.isEnabledFor(logging.WARNING):
            self._log(logging.WARNING, msg, args, **kwargs)

    def error(self, msg, *args, **kwargs):
        """
        Log 'msg % args' with severity 'ERROR'.

        To pass exception information, use the keyword argument exc_info with
        a true value, e.g.

        logger.error("Houston, we have a %s", "major problem", exc_info=1)
        """

        msg, args = reset_msg(msg, *args)
        if self.isEnabledFor(logging.ERROR):
            self._log(logging.ERROR, msg, args, **kwargs)

    def exception(self, msg, *args, exc_info=True, **kwargs):
        """
        Convenience method for logging an ERROR with exception information.
        """
        msg, args = reset_msg(msg, *args)
        self.error(msg, *args, exc_info=exc_info, **kwargs)

    def critical(self, msg, *args, **kwargs):
        """
        Log 'msg % args' with severity 'CRITICAL'.

        To pass exception information, use the keyword argument exc_info with
        a true value, e.g.

        logger.critical("Houston, we have a %s", "major disaster", exc_info=1)
        """
        msg, args = reset_msg(msg, *args)
        msg = "! " * 5 + msg
        if self.isEnabledFor(logging.CRITICAL):
            self._log(logging.CRITICAL, msg, args, **kwargs)


def logger_class_decorator():
    def wrapper(obj):
        obj.debug = LoggerTemplate.debug
        obj.info = LoggerTemplate.info
        obj.warning = LoggerTemplate.warning
        obj.error = LoggerTemplate.error
        obj.exception = LoggerTemplate.exception
        obj.critical = LoggerTemplate.critical
        return obj

    return wrapper


@logger_class_decorator()
class MyLogger(logging.Logger):
    pass


logger = get_my_logger(DOCUMENTPATH, "logging")

if __name__ == "__main__":
    # logger.debug('s')
    logger.debug(1, 213, 543)
    logger.info(1, 213, 543)
    logger.warning(1, 213, 543)
    logger.exception(1, 213, 543)
    logger.critical(1, 213, 543)



#!/usr/bin/python3
import json
import socket
from threading import Thread
from time import sleep

from common.logging_config import logger


class WebSocketClient:
    def conn_ws(self, host="127.0.0.1", port=5678):
        self.host = host
        self.port = port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect((host, port))
        return self.sock

    def send_msg(self, msg, encoding="utf-8"):
        if isinstance(msg, dict) or isinstance(msg, list):
            msg = json.dumps(msg)
        logger.debug("send msg ", self.host, self.port, msg)
        if not isinstance(msg, bytes):
            msg = msg.encode(encoding)
        self.sock.send(msg)

    def recv_msg(self, encoding="utf-8"):
        while True:
            msg = self.sock.recv(1024)
            msg = msg.decode(encoding)
            logger.debug("recv msg", self.host, self.port, msg)


if __name__ == "__main__":
    client = WebSocketClient()
    client.conn_ws()
    Thread(target=client.recv_msg).start()
    while True:
        client.send_msg("示范法")
        sleep(2)



from copy import deepcopy
from elasticsearch import Elasticsearch
from common.utility.args_parsing import BaseArgsParsing


class ESClient(BaseArgsParsing):
    LIST_KEYWORD = ['must', 'should']

    def conn(self, host=None, username=None, password=None, port=None):
        if username and password:
            self.es = Elasticsearch(f"http://{host}/", port=port, http_auth=(username, password))
        else:
            self.es = Elasticsearch(f"http://{host}/", port=port)
        return self.es

    def transform(self, data):
        body = {}
        for key in data:
            if (not key) or (not data.get(key)):
                continue
            values = data.get(key)
            if key in self.RESERVED_FIELD:
                continue
            itme = key.split("_")
            if len(itme) > 1:
                real_key = "_".join(itme[:-1])
                keyword = itme[-1].lower()
                if keyword in self.SUFFIX_KEYWORD:
                    self.build_suffix_keyword_args(body, real_key, keyword, data.get(key))
                    continue

            page, size = self.get_page_and_size(data)

    def build_suffix_keyword_args(self, body, key, keyword, value):
        if keyword == self.GT or keyword == self.GTE or keyword == self.LT or keyword == self.LTE:
            if 'query' in body.keys():
                pass
            else:
                body['query'] = {
                    "range": {
                        key: {
                            "gt": "2021-07-26T10:00:00Z",
                            'lt': "2021-07-26T10:02:00Z",
                        }
                    }
                }

    def build(self, body, keys, field, vaule):
        if body is None:
            body = {}
        if '.' in keys:
            itme = keys.split('.')
            key = itme[0]
            if isinstance(body, dict):
                if key in body.keys():
                    body[key] = self.build(body[key], '.'.join(itme[1:]), field, vaule)
                    return body
                else:
                    child_body = [] if key in self.LIST_KEYWORD else {}
                    body[key] = self.build(child_body, '.'.join(itme[1:]), field, vaule)
                    return body
            else:
                body.append({key: self.build({}, '.'.join(itme[1:]), field, vaule)})
                return body

        else:
            key = keys
            if isinstance(body, dict):
                if key in body.keys():
                    body[key][field] = vaule
                    return body[key]
                else:
                    return {
                        key: {
                            field: vaule
                        }
                    }
            else:
                body.append({key: {field: vaule}})
                return body

    def build2(self, body, keys, field, vaule):
        if body is None:
            body = {}
        if '.' in keys:
            itme = keys.split('.')
            key = itme[0]
            if key in body.keys():
                body[key] = self.build(body[key], '.'.join(itme[1:]), field, vaule)
                return body
            else:
                child_body = [] if key in self.LIST_KEYWORD else {}
                body[key] = self.build(child_body, '.'.join(itme[1:]), field, vaule)
                return body

        else:
            key = keys
            if isinstance(body, dict):
                if key in body.keys():
                    body[key][field] = vaule
                    return body[key]
                else:
                    return {
                        key: {
                            field: vaule
                        }
                    }
            else:
                body.append({key: {field: vaule}})
                return body

    def sort_(self, key, value, body=None):
        if not body:
            body = {}
        sort_data = {
            key: {
                'order': value
            }
        }
        if 'sort' in body.keys():
            body['sort'].append(sort_data)
        else:
            body['sort'] = [sort_data]
        return body

    def and_(self, key, value, body=None):

        body = self.build(body, 'query.bool.must.match_phrase', key, value)
        return body

    def or_(self, key, value, body=None):

        body = self.build(body, 'query.bool.should.match_phrase', key, value)
        return body

    def range_(self, key, kw, value, body=None):
        body = self.build(body, 'query.bool.must.range.' + key, kw, value)
        return body

    def not_(self, key, value, body=None):
        body = self.build(body, 'query.bool.must_not.match_phrase', key, value)
        return body


if __name__ == '__main__':
    es_client = ESClient()
    body = es_client.and_('series', 'P')



import json
import socket
from queue import Queue
from threading import Thread, Lock, Event
from time import sleep
from common.logging_config import logger


class WebSocketServer:
    def __init__(self):
        self.conn_event = {}

    def send_msg(self, conn, msg, encoding="utf-8"):
        """
        发送数据
        :param conn:
        :param msg:
        :param encoding:
        :return:
        """
        if isinstance(msg, dict) or isinstance(msg, list):
            msg = json.dumps(msg)
        # logger.debug("send msg ", msg, conn)
        if not isinstance(msg, bytes):
            msg = msg.encode(encoding)
        try:
            conn.send(msg)
            return True
        except:
            return False

    def recv_msg(self, conn, event: Event, encoding="utf-8"):
        """
        接收客户端发送过来的msg
        :param conn:
        :param encoding:
        :return:
        """
        while event.is_set():
            try:
                data = conn.recv(1024)
            except:
                break
            data = data.decode(encoding)
            logger.info(data)
            # self.send_msg(conn, 'recv' + data)

        logger.info("close recv msg", conn)

    def close_recv_thread(self, conn):
        if conn in self.conn_event.keys():
            event = self.conn_event[conn]
            event.clear()
            del self.conn_event[conn]
            logger.info("close recv thread", conn)
        else:
            logger.warning("non existent key", conn)

    def wait_socket_connect(self):
        """
        循环等待客户端建立连接，并开启后台接收数据
        """
        while True:
            try:
                conn, addr = self.sock.accept()
                logger.debug("连接地址: %s" % str(addr))
                conn.send("connent websocket success".encode("utf-8"))
                event = Event()
                _thread = Thread(target=self.recv_msg, args=(conn, event))
                event.set()
                self.conn_event[conn] = event
                _thread.start()
            except Exception as e:
                logger.exception(e)

    def start_socket_server(self, port=5678, maxwaituser=5):
        """
        socket服务端监听客户端
        1.启动socket
        2.开启等待客户端建立连接的线程
        """
        logger.info("start websocket server", port, maxwaituser)
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        # 绑定端口号
        self.sock.bind(("0.0.0.0", port))

        # 设置最大连接数，超过后排队
        self.sock.listen(maxwaituser)

        # 启线程循环等待客户端建立连接
        Thread(target=self.wait_socket_connect).start()

    def daemon_run_ws_server(self, port=5678, maxwaituser=5):
        Thread(target=self.start_socket_server, args=(port, maxwaituser)).start()


class MeasWSServer(WebSocketServer):
    def __init__(self):
        super().__init__()
        # 保持连接的信息
        self.conn_info = {}

        # 将需要推送的信息入队
        self.q = Queue()

        self.lock = Lock()

    def enqueue_meas(self, topic, meas):
        """
        需要发送的测量数据入队
        :param topic:
        :param meas:
        :return:
        """
        if topic and meas and topic in self.conn_info.keys():
            self.q.put((topic, meas))

    def recv_msg(self, conn, event: Event, encoding="utf-8"):
        """
        重写，接收客户端的msg，订阅数据
        :param conn:
        :param encoding:
        :return:
        """
        while event.is_set():
            try:
                data = conn.recv(1024)
            except:
                break
            data = data.decode(encoding)
            logger.info(data)
            if data in self.conn_info.keys():
                self.conn_info[data].add(conn)
            else:
                self.conn_info[data] = {conn}
            # logger.debug(self.conn_info.keys())
            self.send_msg(conn, "subscrib {data} success".format(data=data))
        logger.info("close recv msg", conn)

    def clear_empty_conn_topic(self):
        """
        清除没有用户连接的topic
        """
        logger.info("start clear empty conn topic thread")
        while True:
            try:
                # 添加没有连接的topic
                empty_list = []
                for topic in self.conn_info:
                    if len(self.conn_info[topic]) == 0:
                        empty_list.append(topic)

                # region 删除当前没有连接的topic
                self.lock.acquire()
                for topic in empty_list:
                    try:
                        if len(self.conn_info[topic]) == 0:
                            del self.conn_info[topic]
                            logger.info("clear empty conn topic", topic)
                    except Exception as e:
                        logger.exception(e)
                self.lock.release()
                # endregion
            except Exception as e:
                logger.exception(e)

            sleep(60)

    def daemon_push_msg(self):
        """
        消息推送到对应的客户端
        """
        logger.info("start daemon push msg thread")
        while True:
            try:
                # region 判断是否有客户端连接，有才推送消息
                if not self.q.empty():
                    data = self.q.get()
                    topic = data[0]
                    if topic in self.conn_info.keys():
                        remove_set = set()
                        for conn in self.conn_info[topic]:
                            status = self.send_msg(conn, data[1])
                            if status == False:
                                # 删除断开连接的记录
                                remove_set.add(conn)
                                # self.conn_info[topic].remove(conn)
                                # logger.info("delete a closed connection", topic, conn)
                        if remove_set:
                            self.conn_info[topic] = self.conn_info[topic] - remove_set
                            logger.info("remove close conn", remove_set, self.conn_info)
                # endregion
            except Exception as e:
                logger.exception(e)

    def daemon_run_server(self, port=5678, maxwaituser=5):
        """
        websockets推送测量数据主线程
        1、启动后台清除空topic（无客户端连接）的线程
        2、启动websockets服务器
        3、启动后台推送测量数据的线程
        :param port:
        :param maxwaituser:
        :return:
        """
        Thread(target=self.clear_empty_conn_topic).start()
        self.daemon_run_ws_server(port=port, maxwaituser=maxwaituser)
        Thread(target=self.daemon_push_msg).start()


if __name__ == "__main__":
    # web = WebSocketServer()
    #
    # web.daemon_run_ws_server()
    web = MeasWSServer()
    web.daemon_run_server()



import math
from dataclasses import dataclass, field, is_dataclass, asdict
from common.logging_config import logger
from django.core.exceptions import FieldError
from django.core.serializers.json import DjangoJSONEncoder
from django.http import JsonResponse
from rest_framework.exceptions import ValidationError



SUCCESS = "success"
FAILURE = "failure"

# # 请求成功，但条件筛选结果集为空
# EMPTY_RESOURCE = 20004

# 请求错误码（主要是参数校验）
REQUEST_ERROR_CODE = 400


# 未授权
UNAUTHORIZED = 401

#
FORBID_CODE = 403

# 服务器内部错误
SERVER_ERROR_CODE = 500

# 上层服务错误
UPSTREAM_SERVER_ERROR_CODE = 502

# 请求上层服务超时
UPSTREAM_SERVER_WAIT_TIMEOUT_CODE = 504

ERROR_CODE_TUPLE = (
    REQUEST_ERROR_CODE,
    FORBID_CODE,
    SERVER_ERROR_CODE,
    UPSTREAM_SERVER_ERROR_CODE,
    UPSTREAM_SERVER_WAIT_TIMEOUT_CODE,
)
results_version: str = ""


def init_version(version: str):
    global results_version
    results_version = version


def get_error_status_code(error):
    """
    根据错误类型，获取状态码和信息
    :param error:
    :return:
    """
    logger.exception(error)
    if isinstance(error, ValidationError):
        status_code = REQUEST_ERROR_CODE
    elif isinstance(error, ValueError):
        status_code = REQUEST_ERROR_CODE
    elif isinstance(error, FieldError):
        status_code = REQUEST_ERROR_CODE
    else:
        status_code = SERVER_ERROR_CODE
    describe = str(error)
    return status_code, describe


@dataclass
class Results:
    status: str = FAILURE
    code: int = REQUEST_ERROR_CODE
    version: str = results_version
    describe: str = ""
    data_length: int = 0
    data: dict = field(default_factory=dict)

    def __setattr__(self, key, value):
        super(Results, self).__setattr__(key, value)
        if key == "code":
            if value not in ERROR_CODE_TUPLE:
                super(Results, self).__setattr__("status", SUCCESS)
            else:
                super(Results, self).__setattr__("status", FAILURE)
        if key == "data":
            if (isinstance(value, dict) or isinstance(value, list)) and self.data_length == 0:
                super(Results, self).__setattr__("data_length", len(value))
        if not self.version:
            super(Results, self).__setattr__("version", results_version)


@dataclass
class PageTypeResults:
    result: list = field(default_factory=list)
    total_page: int = 0
    total: int = 0
    page: int = 0
    size: int = 0
    group: list = field(default_factory=list)

    def __setattr__(self, key, value):
        if key != "result" and key != "group":
            value = int(value)
        super(PageTypeResults, self).__setattr__(key, value)
        if all([self.total, self.page, self.size]) and self.total_page == 0:
            super(PageTypeResults, self).__setattr__("total_page", math.ceil(self.total / self.size))


class RESTResponse(JsonResponse):
    def __init__(self, data, encoder=DjangoJSONEncoder, safe=True, json_dumps_params=None, **kwargs):
        if not json_dumps_params:
            json_dumps_params = {"ensure_ascii": False}
        if isinstance(data, Results):
            super(RESTResponse, self).__init__(
                asdict(data),
                status=data.code,
                encoder=encoder,
                safe=safe,
                json_dumps_params=json_dumps_params,
                **kwargs,
            )
        elif is_dataclass(data):

            super(RESTResponse, self).__init__(
                asdict(data), encoder=encoder, safe=safe, json_dumps_params=json_dumps_params, **kwargs
            )
        else:
            super(RESTResponse, self).__init__(
                data, encoder=encoder, safe=safe, json_dumps_params=json_dumps_params, **kwargs
            )



from common.rest_extend.response import RESTResponse, Results, FORBID_CODE
from django.utils.deprecation import MiddlewareMixin


class AuthMiddleware(MiddlewareMixin):
    def process_request(self, request):
        path = request.path
        if "login" not in path and path != "/":
            account = request.session.get("account", None)
            results = Results()
            if not account:
                results.describe = "no permission!!!"
                results.code = FORBID_CODE
                return RESTResponse(results)
            authorization = self.permissions(request, account)
            if not authorization:
                results.describe = "no permission!!!"
                results.code = FORBID_CODE
                return RESTResponse(results)

    def permissions(self, request, account):
        from tenant_auth.models import AuthTenant, AuthTenantRole, AuthPermission, AuthRolePermissions

        method = request.method
        path = request.path
        tenant = AuthTenant.objects.filter(account=account).first()
        if tenant:
            if tenant.account == 'admin':
                return True
            permission = AuthPermission.objects.filter(action=method, source_id=path).first()
            if permission:
                role_permissions = AuthRolePermissions.objects.filter(permission_id=permission.name).first()
                if role_permissions:
                    tenant_role = AuthTenantRole.objects.filter(
                        role_id=role_permissions.role_id, tenant_id=account
                    ).first()
                    if tenant_role:
                        return True

        return False



# coding=utf-8

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='common-code',

    version="0.5.5",
    description=(
        ''

    ),
    long_description=open('README.md', 'r').read(),
    long_description_content_type="text/markdown",
    author='FengJP',
    author_email='852497515@qq.com',
    maintainer='FengJP',
    maintainer_email='852497515@qq.com',
    license='MIT License',
    packages=find_packages(),
    platforms=["ubuntu", 'windows'],
    url='http://git.runjian.com:18080/FengJingPing/common',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'Django',
        'djangorestframework',
        'CMRESHandler',
        'requests',
        'gunicorn',
        'paho-mqtt',
        'pika',
    ]
)



from dataclasses import asdict

from common.rest_extend.response import Results, get_error_status_code, RESTResponse


def vaild_decorator(obj=None, obj_serializer=None, partial=False):
    """
    参数验证装饰器（这里当 middleware ，验证失败不往下执行，直接返回Response）
    :param obj: 实体类
    :param obj_serializer: 需要校验的序列化类
    :param partial: 是否部分校验
    :return:
    """

    def external_wrapper(function):
        def internal_wrapper(obj_request, request, *args, **kwargs):
            """

            :param obj_request: 被装饰的实体类
            :param request:
            :param args:
            :param kwargs:
            :return:
            """
            try:
                if request.method == "GET":
                    data = request.GET
                else:
                    data = request.data
                serializer = obj_serializer(data=data, partial=partial)
                valid = serializer.is_valid(raise_exception=True)
                kwargs["obj_ser"] = serializer
            except Exception as e:
                results = Results()
                results.describe = str(e)
                status_code, describe = get_error_status_code(e)
                return RESTResponse(asdict(results), status=status_code)
            return function(obj_request, request, *args, **kwargs)
            # response = function(obj_request, request, *args, **kwargs)
            # if json.loads(response.content).get('status') != SUCCESS:
            #     response.status = SERVER_ERROR_CODE
            # return response

        return internal_wrapper

    return external_wrapper


def view_decorator(function):
    """
    视图装饰器
    :param function:
    :return:
    """

    def wrapper(request, *args, **kwargs):
        return function(request, *args, **kwargs)

    return wrapper


def relevance_vaild_legal_decorator(func):
    def wrapper(obj, request, *args, **kwargs):
        group = request.GET.get("group")
        if group:
            results = Results()
            results.describe = "此接口不支持参数 'group'！！！"
            return RESTResponse(results)
        return func(obj, request, *args, **kwargs)

    return wrapper



from django.db import models


# Create your models here.
# from vendor_app.models import Vendor


class BaseTenant(models.Model):
    class Meta:
        db_table = "tenant"
        abstract = True

    tenant_name = models.CharField(null=False, max_length=16, help_text="租户名称", unique=True)
    type = models.CharField(null=False, max_length=16, help_text="租户类型")
    iot_platform_name = models.CharField(null=True, max_length=16, help_text="iot名称")
    db = models.CharField(null=True, max_length=16, help_text="数据库")
    es = models.CharField(null=True, max_length=16, help_text="es")
    account = models.CharField(null=False, max_length=16, help_text="账号", unique=True)
    password = models.CharField(null=False, max_length=16, help_text="密码")
    user_name = models.CharField(null=False, max_length=16, help_text="用户名称", unique=True)
    creator = models.CharField(max_length=16, help_text="创建人", null=True)
    addon = models.DateTimeField(auto_now_add=True, help_text="添加时间")
    update = models.DateTimeField(auto_now=True, help_text="最后更新时间")


class AuthTenant(BaseTenant):
    class Meta:
        db_table = "tenant"
        managed = False


class BaseSource(models.Model):
    class Meta:
        db_table = "source"
        abstract = True

    name = models.CharField(max_length=16, null=False, help_text="名称")
    uri = models.CharField(max_length=32, null=False, help_text="uri", unique=True)


class AuthSource(BaseSource):
    class Meta:
        db_table = "source"
        managed = False


class BasePermission(models.Model):
    class Meta:
        db_table = "permission"
        unique_together = ("source", "action")
        abstract = True

    name = models.CharField(max_length=16, null=False, help_text="权限名称", unique=True)

    action = models.CharField(
        max_length=8, choices=(("GET", "GET"), ("POST", "POST"), ("PUT", "PUT"), ("DELETE", "DELETE")), help_text="资源操作"
    )
    source = models.ForeignKey(to=AuthSource, to_field="uri", on_delete=models.SET_NULL, null=True)


class AuthPermission(BasePermission):
    class Meta:
        db_table = "permission"
        unique_together = ("source", "action")
        managed = False


class BaseRole(models.Model):
    class Meta:
        db_table = "role"
        abstract = True

    name = models.CharField(max_length=16, null=False, unique=True, help_text="名称")


class AuthRole(BaseRole):
    class Meta:
        db_table = "role"
        managed = False


class BaseRolePermissions(models.Model):
    class Meta:
        db_table = "role_permissions"
        abstract = True

    role = models.ForeignKey(to=AuthRole, on_delete=models.SET_NULL, to_field="name", null=True)
    permission = models.ForeignKey(to=AuthPermission, to_field="name", on_delete=models.SET_NULL, null=True)


class AuthRolePermissions(BaseRolePermissions):
    class Meta:
        db_table = "role_permissions"
        managed = False


class BaseTenantRole(models.Model):
    class Meta:
        db_table = "tenant_role"
        abstract = True

    tenant = models.ForeignKey(to=AuthTenant, to_field="account", null=True, on_delete=models.SET_NULL)
    role = models.ForeignKey(to=AuthRole, to_field="name", on_delete=models.SET_NULL, null=True)


class AuthTenantRole(BaseTenantRole):
    class Meta:
        db_table = "tenant_role"
        managed = False



import json
from queue import Queue
from threading import Thread

from django.utils.deprecation import MiddlewareMixin

from common.logging_config import logger


class EventMiddleware(MiddlewareMixin):
    def __init__(self, get_response=None):
        super().__init__(get_response)
        self.q = Queue()
        Thread(target=self.send_event).start()

    def process_response(self, request, response):
        if response.status_code < 300 and request.method != 'GET':
            self.parsing(request, response)
        return response

    def parsing(self, request, response):
        data = json.loads(request.body)
        self.q.put(data)

    def send_event(self):
        while True:
            try:
                body = self.q.get()


            except Exception as e:
                logger.exception(e)
